Package: Xplortext
Type: Package
Encoding: UTF-8
Title: Statistical Analysis of Textual Data
Version: 1.6.1
Date: 2026-06-15
Authors@R: c(
            person(given="Ramón",family="Alvarez-Esteban", email="ramon.alvarez@unileon.es", role = c("aut", "cre"), comment = c(ORCID="0000-0002-4751-2797")),
            person(given="Mónica",family="Bécue-Bertaut", email="monica.becue@upc.edu", role = "aut", comment = c(ORCID = "0000-0002-6027-3655")),
            person(given="Josep-Anton",family="Sánchez-Espigares", email="josep.a.sanchez@upc.edu",role="ctb",  comment = c(ORCID = "0000-0001-8195-1913")),
            person(given="Belchin Adriyanov", family="Kostov", email="belchin.adriyanov.kostov@upc.edu",role="ctb", comment = c(ORCID ="0000-0002-2126-3892")))
Maintainer: Ramón Alvarez-Esteban <ramon.alvarez@unileon.es>
Author: Ramón Alvarez-Esteban [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4751-2797>),
  Mónica Bécue-Bertaut [aut] (ORCID:
    <https://orcid.org/0000-0002-6027-3655>),
  Josep-Anton Sánchez-Espigares [ctb] (ORCID:
    <https://orcid.org/0000-0001-8195-1913>),
  Belchin Adriyanov Kostov [ctb] (ORCID:
    <https://orcid.org/0000-0002-2126-3892>)
Description: Provides a set of functions devoted to multivariate exploratory statistics on textual data. Classical methods such as correspondence analysis and agglomerative hierarchical clustering are available. Chronologically constrained agglomerative hierarchical clustering enriched with labelled-by-words trees is offered. Given a division of the corpus into parts, their characteristic words and documents are identified. Further, accessing to 'FactoMineR' functions is very easy. Two of them are relevant in textual domain. MFA() addresses multiple lexical table allowing applications such as dealing with multilingual corpora as well as simultaneously analyzing both open-ended and closed questions in surveys. See <http://xplortext.unileon.es> for examples.
License: GPL (>= 2.0)
Depends: R (>= 4.6), FactoMineR(>= 2.15), ggplot2(>= 4.0.3), tm(>=
        0.7-18)
Imports: ape, cluster, dendextend, flexclust, flashClust, ggdendro,
        ggforce, ggpubr, ggrepel, graphics, gridExtra, MASS, methods,
        patchwork, plotly, stringi, stringr, slam, stats, utils, vegan
URL: https://xplortext.unileon.es
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-15 17:23:58 UTC; ralve
Repository: CRAN
Date/Publication: 2026-06-15 18:00:09 UTC
Built: R 4.6.0; ; 2026-06-15 20:00:02 UTC; unix
