S3method(plot,LexCA)
S3method(plot,LexCHCca)
S3method(plot,LexChar)
S3method(plot,LexGalt)
S3method(plot,LexHCca)
S3method(plot,TextData)
S3method(print,LexCA)
S3method(print,LexChar)
S3method(print,TextData)
S3method(summary,LexCA)
S3method(summary,LexChar)
S3method(summary,TextData)
export(LabelTree)
export(LexCA)
export(LexCHCca)
export(LexChar)
export(LexGalt)
export(LexHCca)
export(TextData)
export(ellipseLexCA)
export(summary.LexCA)
export(summary.LexChar)
export(summary.TextData)
export(plot.TextData)
export(plot.LexCA)
export(plot.LexHCca)
export(plot.LexCHCca)
export(plot.LexGalt)
export(plot.LexChar)
import(FactoMineR)
import(ape, except = c(rotate, ladderize))
import(cluster)
import(dendextend, except=c(cutree))
import(flashClust)
import(flexclust, except = c(barplot, boxplot, predict, plot, pairs, image))
import(ggdendro, except = c(theme_dendro))
import(ggforce)
import(ggplot2)
import(ggpubr, except = c(rotate))
import(ggrepel)
import(graphics)
import(gridExtra)
import(patchwork)
import(slam)
import(stats, except = c(hclust))
import(stringi)
import(stringr)
import(tm)
import(utils)
importFrom(MASS,ginv)
importFrom(ggforce,geom_mark_ellipse)
importFrom(ggforce,geom_mark_hull)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,adjustcolor)
importFrom("grDevices", "dev.cur", "dev.off", "graphics.off")
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,dev.new)
importFrom(grDevices,palette)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,barplot)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,locator)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(methods,hasArg)
importFrom(plotly,plotly_build)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,write.table)
importFrom(vegan,mantel)
